package Magnetic "Library for electromagnetic actuators and drives based on magnetic flux tubes"


annotation (
  Coordsys(
    extent=[-100, -100; 100, 100],
    grid=[2, 2],
    component=[20, 20]),
  Window(
    x=0.45,
    y=0.01,
    width=0.44,
    height=0.65,
    library=1,
    autolayout=1),
  Documentation(info="<html>
<br>
This library contains components to model magnetic actuators and drives, e.g. eletro-magneto-mechanical linear actuators or rotational electrical machines. In these devices, magnetic fields imposed by coils and/or permanent magnets are utilised for generation of a force respectively torque. For modeling in this library, the conept of <i>magnetic flux tubes</i> is utilised  where distribution of the magnetic field inside an actuator is described with a <i>magnetic network</i>. These network models are suited for both coarse design of the magnetic subsystem of actuators as well as for dynamic simulation  at system level together with neighboring subsystems. Currently, the library contains the following sublibraries:
<ul>
<li><b>Interfaces: </b>Definition of connectors and a magnetic flux tube partial model</li>
<li><b>Reluctance: </b>Basic flux tube geometries and selected reluctance elements for stray flux through air - to be extended</li>
<li><b>Sources: </b>Sources of a magnetic potential difference</li>
<li><b>Material: </b>Material properties of ferromagnetic and permanent magnetic materials - to be extended</li>
<li><b>Solenoid: </b>Model(s) of electromagnetic actuator(s) - to be extended</li>
<li><b>Examples: </b>Exemplary usage of actuator models</li>
</ul>

</p>
<br>
<h3>Overview on the Conept of Magnetic Flux Tubes</h3>
<p>
Following below, the concept of magnetic flux tubes is outlined in short. For a detailed description of flux tube elements, please have a look at the literature listed in the References section.
</p>
<p>
Magnetic flux tubes enable for modeling of magnetic fields with networks. The figure below and the following equations illustrate the transition from the original magnetic field quantities described by <i>Maxwell</i>'s equations to network elements with a flow variable and an across variable:
<br>
<dl>
<dd>
<p><IMG SRC=\"../Images/magnetic_flux_tube_schematic.png\" ALT=\"Magnetic flux tube\"></p>
</dd>
</dl>
<br>
For a region with an approximately homogeneous distribution of the magnetic field strength <b>H</b> and the magnetic flux density <b>B</b> through cross sectional area <i>A</i> at each length coordinate <i>s</i>  (<i>A</i> perpendicular to the direction of the magnetic field lines), a magnetic reluctance <i>R<sub>mag</sub></i> can be defined:<br>
<dl>
<dd>
<p><IMG SRC=\"../Images/equation_3.png\" ALT=\"Transition from field quantities to flow- and across variables\"></p>
</dd>
</dl>
With the definition of a magnetic potential difference <i>V<sub>mag</sub></i> as across variable and the magnetic flux <i><font face=\"Symbol\">F</font></i> as flow variable, a reluctance element <i>R<sub>mag</sub></i> can be defined analogue to resistive network elements in other physical domains. With <i>Maxwell</i>'s constitutive equation<br>
<dl>
<dd>
<p><IMG SRC=\"../Images/equation_5.png\" ALT=\"Maxwell's constitutive equation\"></p>
</dd>
</dl>
the general formula for calculation of a magnetic reluctance <i>R<sub>mag</sub></i> from its gemotetry and material properties is:<br>
<dl>
<dd>
<p><IMG SRC=\"../Images/equation_6.png\" ALT=\"General formula for calculation of a magnetic reluctance\"></p>
</dd>
</dl>
For a prismatic or cylindrical volume of length <i>l</i> and cross sectional area <i>A</i> with the magnetic flux entering and leaving the region through its end planes, the above equation simplifies to:<br>
<dl>
<dd>
<p><IMG SRC=\"../Images/equation_7.png\" ALT=\"Magnetic reluctance of a prismatic or cylindrical volume\"></p>
</dd>
</dl>
<p>
Similar equations can be derived for other geometric shapes. Network elements for sources of a magnetic potential difference <i>V<sub>mag</sub></i>, e.g. coils or permanent magnets can be given, too. 
</p>
<p>
The resulting network models for actuators reflect the main dimensions of the device as well as its  material properties. They are hence suited for coarse design of the geometry of magnetic actuators. 
Due to the little computational effort of the magnetic networks, dynamic behavior of the devices can be simulated during system design efficiently. 
</p>
<hr>
<br>
<h4>References</h4>
<p>
<dl>
<dt>
A first realisation of this Magnetic library is described in:</dt>
<dd>
<p><i>B&ouml;drich, Th.; Roschke, Th.</i>: A Magnetic Library for Modelica. Proceedings of the 4<sup>th</sup> International Modelica Conference. Hamburg, Germany, March 7-8, 2005</p>
</dd>
<dt>
The method of magnetic flux tubes as well as derivation of many flux tube geometries is explained in detail in:</dt>
<dd>
<p><i>Roters, H.</i>: Electromagnetic Devices. New York: John Wiley & Sons 1941.</p>
</dd>
<dt>
Structure, properties, applications and design of translational electromagnetic actuators are thoroughly described in:</dt>
<dd>
<p><i>Kallenbach, E.; Eick, R.; Quendt, P.; Str&ouml;hla, T.; Feindt, K.; Kallenbach, M.</i>: Elektromagnete: Grundlagen, Berechnung, Entwurf und Anwendung. 2. Aufl. Wiesbaden: B.G. Teubner 2003   and</p>
</dd>
<dd>
<p><i>Roschke, Th.</i>: Entwurf geregelter elektromagnetischer Antriebe f&uuml;r Luftsch&uuml;tze. D&uuml;sseldorf: VDI-Verlag 2000.</p>
</dd>
<dt>
Application of the method of magnetic flux tubes for the design of rotational electrical machines is explained in:</dt>
<dd>
<p><i>Hendershot, J.R. Jr.; Miller, T.J.E.</i>: Design of Brushless Permanent-Magnet Motors. Magna Physics Publishing and Oxford University Press 1994.</p>
</dd>
</dl>
</p>
<hr>
<br>
This first version of the Magnetic library is not yet complete. Improvements and add-ons are planned for future releases, as for example: 
<ul>
<li>additional flux tube geometries, especially reluctance elements for stray flux through air,</li>
<li>provision of pre-defined air gap models with both working flux and stray flux for different pole shapes, </li>
<li>estimation of eddy current losses in flux tube elements with electrical conductivity and</li>
<li>more versatile and refined description of material data, e.g. with B-splines.</li>
</ul>
Also note that composite models are not restricted to solenoid actuators. Models for different actuator types such as electrodynamic actuators (voice coils) or rotational electrical machines can be built with the reluctance and source elements of this library, too.
<br>
<br>
<hr>
<br>
<dl>
<dt>
<b>Main Author:</b></dt>
<br>
<dd>
    Thomas B&ouml;drich<br>
    Dresden University of Technology<br>
    Institute of Electromechanical and Electronic Design<br>
    01062 Dresden, Germany<br>
    Phone: ++49 - 351 - 463 36296<br>
    Fax: ++49 - 351 - 463 37183<br>
    email: <A HREF=\"mailto:Thomas.Boedrich@mailbox.tu-dresden.de\">Thomas.Boedrich@mailbox.tu-dresden.de</A><br></dd>
</dl>
<br>
<br>
<dl>
<dt>
<b>Copyright:</b></dt>
<br>
<dd>
Copyright (C) 2005, Modelica Association and Thomas B&ouml;drich.<br>
<br>
<i>The Modelica package is <b>free</b> software; it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package 
Modelica in file \"Modelica/package.mo\".</i></dd>
</dl>
<br>
</HTML>
"),
  Icon(
    Rectangle(extent=[36, 2; 60, -84], style(color=0)),
    Line(points=[20, 2; -60, 2; -60, -84; 20, -84; 20, -62; -36, -62; -36, -20;
           20, -20; 20, 2], style(color=0)),
    Line(points=[-64, -22; -32, -30], style(color=0)),
    Line(points=[-64, -30; -32, -38], style(color=0)),
    Line(points=[-64, -38; -32, -46], style(color=0)),
    Line(points=[-64, -46; -32, -54], style(color=0)),
    Line(points=[-64, -54; -32, -62], style(color=0)),
    Line(points=[-64, -14; -32, -22], style(color=0)),
    Line(points=[-64, -14; -78, -14], style(color=0)),
    Line(points=[-64, -54; -78, -54], style(color=0)),
    Ellipse(extent=[-78, -18; -86, -10], style(color=0)),
    Ellipse(extent=[-78, -58; -86, -50], style(color=0))),
    version="0.9",
    conversion(noneFromVersion=""),
    uses(Modelica(version="2.1")));

  import SI = Modelica.SIunits;


  extends Modelica.Icons.Library2;


  package Interfaces "Interfaces of magnetic equivalent networks" 
    extends Modelica.Icons.Library;
    package SI = Modelica.SIunits;
  
    annotation (Coordsys(
        extent=[-100, -100; 100, 100],
        grid=[2, 2],
        component=[20, 20]), Window(
        x=0,
        y=0.48,
        width=0.2,
        height=0.21,
        library=1,
        autolayout=1));
  
    connector MagneticPort "Magnetic connector" 
      SI.MagneticPotentialDifference V_mag "Magnetic potential at the port";
      flow SI.MagneticFlux Phi "Magnetic flux flowing into the port";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Rectangle(extent=[-100, -100; 100, 100], style(color=45,
                fillColor=45)), Text(
            extent=[-76, -102; 80, -164],
            string="%name",
            style(color=45))),
        Icon(Rectangle(extent=[-100, 100; 100, -100], style(color=45, fillColor=
                  45)), Text(
            extent=[-100, 112; 100, 164],
            string="%name",
            style(color=45))),
        Window(
          x=0.31,
          y=0.08,
          width=0.39,
          height=0.52));
    end MagneticPort;
  
    connector PositiveMagneticPort 
      SI.MagneticPotentialDifference V_mag "Magnetic potential at the port";
      flow SI.MagneticFlux Phi "Magnetic flux flowing into the port";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Rectangle(extent=[-100, 100; 100, -100], style(color=45,
                fillColor=45))),
        Icon(Rectangle(extent=[-100, 100; 100, -100], style(color=45, fillColor=
                  45)), Text(
            extent=[-102, 160; 100, 108],
            string="%name",
            style(color=45, fillColor=45))),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49));
    end PositiveMagneticPort;
  
    connector NegativeMagneticPort 
      SI.MagneticPotentialDifference V_mag "Magnetic potential at the port";
      flow SI.MagneticFlux Phi "Magnetic flux flowing into the port";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(Rectangle(extent=[-100, 100; 100, -100], style(color=45,
                fillPattern=0))),
        Icon(Rectangle(extent=[-100, 100; 100, -100], style(
              color=45,
              rgbcolor={255,127,0},
              fillColor=7,
              rgbfillColor={255,255,255})),
                      Text(
            extent=[-102, 156; 100, 110],
            string="%name",
            style(color=45, fillColor=45))),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49));
    end NegativeMagneticPort;
  
    partial model MagneticFluxTube 
    "Component with two magnetic ports p and n and magnetic flux Phi from p to n" 
    
      PositiveMagneticPort p "Positive magnetic port" 
        annotation (extent=[-110, -10; -90, 10]);
      NegativeMagneticPort n "Negative magnetic port" 
        annotation (extent=[90, -10; 110, 10]);
    
      SI.MagneticPotentialDifference V_mag 
      "Magnetic potential difference between the two ports";
      SI.MagneticFlux Phi "Magnetic flux flowing from pin p to pin n";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Documentation(info="<HTML>
<P>
Partial model of a flux tube component with <b>two</b> magnetic ports:
the positive port connector <i>p</i>, and the negative port
connector <i>n</i>. It is assumed that the magnetic flux flowing
into pin p is identical to the flux flowing out of pin n.
This magnetic flux is provided explicitly as flux Phi.
</P>
</HTML>
"),     Diagram(
          Line(points=[-110, 20; -85, 20], style(color=9, fillColor=9)),
          Polygon(points=[-95, 23; -85, 20; -95, 17; -95, 23], style(
              color=9,
              fillColor=9,
              fillPattern=1)),
          Line(points=[90, 20; 115, 20], style(color=9, fillColor=9)),
          Line(points=[-125, 0; -115, 0], style(color=9)),
          Line(points=[-120, -5; -120, 5], style(color=9)),
          Text(
            extent=[-110, 25; -90, 45],
            string="Phi",
            style(color=9)),
          Polygon(points=[105, 23; 115, 20; 105, 17; 105, 23], style(
              color=9,
              fillColor=9,
              fillPattern=1)),
          Line(points=[115, 0; 125, 0], style(color=9)),
          Text(
            extent=[90, 45; 110, 25],
            string="Phi",
            style(color=9))),
        Window(
          x=0.33,
          y=0.04,
          width=0.63,
          height=0.67),
        Icon);
    
    equation 
      V_mag = p.V_mag - n.V_mag;
      0 = p.Phi + n.Phi;
      Phi = p.Phi;
    end MagneticFluxTube;
  end Interfaces;


  package Reluctance "Magnetic flux tube elements for magnetic networks" 
    extends Modelica.Icons.Library;
    package SI = Modelica.SIunits;
  
    annotation (Coordsys(
        extent=[-100, -100; 100, 100],
        grid=[2, 2],
        component=[20, 20]), Window(
        x=0,
        y=0.48,
        width=0.2,
        height=0.21,
        library=1,
        autolayout=1),
      Documentation(info="<html>
<br>
Please refer to the introductory information of the Magnetic library for an explanation of flux tube elements. 
</html>"));
  
    model MagneticGround "Magnetic ground node" 
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Documentation(info="<HTML>
<P>
The default magnetic potential at the magnetic ground node is zero.<br>
Every magnetic equivalent network must contain
at least one magnetic ground object.
</P>
</HTML>
"),     Icon(
          Line(points=[-60, 50; 60, 50], style(color=45)),
          Line(points=[-40, 30; 40, 30], style(color=45)),
          Line(points=[-20, 10; 20, 10], style(color=45)),
          Line(points=[0, 90; 0, 50], style(color=45)),
          Text(
            extent=[-100, -40; 100, -10],
            string="%name",
            style(color=45))),
        Diagram(
          Line(points=[-60, 50; 60, 50], style(color=45, thickness=2)),
          Line(points=[-40, 30; 40, 30], style(color=45, thickness=2)),
          Line(points=[-20, 10; 20, 10], style(color=45, thickness=2)),
          Line(points=[0, 90; 0, 50], style(color=45, thickness=2)),
          Text(
            extent=[-24, -38; 22, -6],
            style(color=45),
            string="p.V_mag=0")),
        Window(
          x=0.23,
          y=0.23,
          width=0.59,
          height=0.63));
      Magnetic.Interfaces.MagneticPort p 
        annotation (extent=[-10, 110; 10, 90], rotation=-90);
    equation 
      p.V_mag = 0;
    end MagneticGround;
  
    model LinearReluctance 
    "Magnetic reluctance of flux tube with constant relative permeability" 
    
      extends Interfaces.MagneticFluxTube;
    
      parameter SI.RelativePermeability my_r = 1 
      "Constant relative permeability";
    
      parameter SI.Length l = 0.1 "Length of flux tube";
      parameter SI.Area A = 1e-4 "Cross sectional area of flux tube";
      SI.MagneticFluxDensity B "Flux density";
      SI.Reluctance R_mag "Magnetic reluctance";
    
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Window(
          x=0.16,
          y=0.15,
          width=0.6,
          height=0.6),
        Icon(
          Rectangle(extent=[-70, 30; 70, -30], style(
              color=45,
              fillColor=7,
              fillPattern=1)),
          Line(points=[-90, 0; -70, 0], style(color=45)),
          Line(points=[70, 0; 90, 0], style(color=45)),
          Text(
            extent=[-100, -78; 100, -36],
            string="%name",
            style(color=45))),
        Diagram);
    equation 
    
      R_mag = l/(Modelica.Constants.mue_0*my_r*A);
      R_mag*Phi = V_mag;
      B = Phi/A;
    
    end LinearReluctance;
  
    model NonlinearReluctance 
    "Magnetic reluctance of a ferromagnetic flux tube with nonlinear B(H) relationship" 
    
      extends Interfaces.MagneticFluxTube;
    
      parameter SI.Length l = 0.01 "Length of flux tube";
      parameter SI.Area A = 1e-4 "Cross sectional area of flux tube";
      SI.MagneticFluxDensity B "Flux density";
      SI.Reluctance R_mag "Magnetic reluctance";
      SI.RelativePermeability my_r "Relative permeability";
    
      replaceable function my_r_Material = 
          Magnetic.Material.Softmagnetic.Steel_9SMn28K 
        annotation(choices(
          choice(redeclare function my_r_Material = 
                Magnetic.Material.Softmagnetic.Steel_9SMn28K "Steel 9SMn28K"),
          choice(redeclare function my_r_Material = 
                Magnetic.Material.Softmagnetic.VacoferS2 
            "Pure Iron Vacofer S2 (RFe20)"),
          choice(redeclare function my_r_Material = 
                Magnetic.Material.Softmagnetic.MUMETALL "Nickle Iron MUMETALL"),
          choice(redeclare function my_r_Material = 
                Magnetic.Material.Softmagnetic.Steel_X6Cr17 
            "Sheet Steel X6Cr17"),
          choice(redeclare function my_r_Material = 
                Magnetic.Material.Softmagnetic.M330_50A 
            "Electric sheet M330 50A")));
    
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Window(
          x=0.16,
          y=0.15,
          width=0.6,
          height=0.6),
        Icon(
          Rectangle(extent=[-70, 30; 70, -30], style(
              color=45,
              fillColor=7,
              fillPattern=1)),
          Line(points=[-90, 0; -70, 0], style(color=45)),
          Line(points=[70, 0; 90, 0], style(color=45)),
          Text(
            extent=[-100, -78; 100, -36],
            string="%name",
            style(color=45)),
          Rectangle(extent=[44,30; 70,-30],   style(
              color=45,
              fillColor=45,
              fillPattern=7))));
    equation 
      my_r = my_r_Material(B);
      R_mag = l/(Modelica.Constants.mue_0*my_r*A);
      R_mag*Phi = V_mag;
      B = Phi/A;
    
    end NonlinearReluctance;
  
    model WorkingAirGap 
    "Magnetic reluctance of a working air gap and generation of a reluctance force" 
    
      extends Interfaces.MagneticFluxTube;
    
      SI.Area A "Cross sectional area of flux tube";
      SI.MagneticFluxDensity B "Flux density";
      SI.Reluctance R_mag "Magnetic reluctance";
    
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Window(
          x=0.16,
          y=0.15,
          width=0.6,
          height=0.6),
        Icon(
          Rectangle(extent=[-70, 30; 70, -30], style(
              color=45,
              fillColor=7,
              fillPattern=1)),
          Line(points=[-90, 0; -70, 0], style(color=45)),
          Line(points=[70, 0; 90, 0], style(color=45)),
          Text(
            extent=[-100, -78; 100, -36],
            string="%name",
            style(color=45))),
        Diagram);
      Modelica.Mechanics.Translational.Interfaces.Flange_a AirGapFlange 
      "Interface between magnetic and mechanical subsystem" 
        annotation (extent=[-10,20; 10,40]);
    equation 
      R_mag = AirGapFlange.s/(Modelica.Constants.mue_0*A);
      R_mag*Phi = V_mag;
      B = Phi/A;
      AirGapFlange.f = Phi * Phi / (2 * Modelica.Constants.mue_0 * A) 
      "Reluctance force calculated with Maxwell's formula";
    
    end WorkingAirGap;
  
    package Air "Flux tube geometries for stray flux through air" 
    
        model CircularHalfCylinder 
      "Magnetic reluctance between the edges of a cylindrical air gap" 
      
          extends Interfaces.MagneticFluxTube;
      
          parameter SI.Length r = 0.01 "Radius of enclosed cylindrical air gap";
          SI.Reluctance R_mag "Magnetic reluctance";
      
          annotation (
            Coordsys(
              extent=[-100, -100; 100, 100],
              grid=[2, 2],
              component=[20, 20]),
            Window(
              x=0.16,
              y=0.15,
              width=0.6,
              height=0.6),
            Icon(
              Rectangle(extent=[-70, 30; 70, -30], style(
                  color=45,
                  fillColor=7,
                  fillPattern=1)),
              Line(points=[-90, 0; -70, 0], style(color=45)),
              Line(points=[70, 0; 90, 0], style(color=45)),
              Text(
                extent=[-100, -78; 100, -36],
                string="%name",
                style(color=45))),
            Diagram);
        equation 
          R_mag = 1 / (1.68 * Modelica.Constants.mue_0 * r);
          R_mag*Phi = V_mag;
      
        end CircularHalfCylinder;
    
        model CircularHalfAnnulus 
      "Magnetic reluctance of a half annulus, geometry is fixed during simulation" 
      
          extends Interfaces.MagneticFluxTube;
      
          parameter SI.Length r = 0.01 "radius of enclosed region";
          parameter SI.Length d = 0.01 "Inner diameter of half annulus";
          parameter SI.Length t = 0.01 "Thickness of half annulus";
          parameter SI.Length t_res = 0.002 
        "Residual thickness of half annulus";
          SI.Reluctance R_mag "Magnetic reluctance";
      
          annotation (
            Coordsys(
              extent=[-100, -100; 100, 100],
              grid=[2, 2],
              component=[20, 20]),
            Window(
              x=0.16,
              y=0.15,
              width=0.6,
              height=0.6),
            Icon(
              Rectangle(extent=[-70, 30; 70, -30], style(
                  color=45,
                  fillColor=7,
                  fillPattern=1)),
              Line(points=[-90, 0; -70, 0], style(color=45)),
              Line(points=[70, 0; 90, 0], style(color=45)),
              Text(
                extent=[-100, -78; 100, -36],
                string="%name",
                style(color=45))),
            Diagram);
        equation 
          R_mag = (d/(t+t_res) + 1) / (Modelica.Constants.mue_0 * r * 4);
          R_mag*Phi = V_mag;
      
        end CircularHalfAnnulus;
    
        model VariableCircularHalfAnnulus 
      "Magnetic reluctance of a half annulus, geometry can vary during simulation" 
      
          extends Interfaces.MagneticFluxTube;
      
          parameter SI.Length r = 0.01 "radius of enclosed region";
          SI.Length d = 0.01 "Inner diameter of half annulus";
          SI.Length t = 0.01 "Thickness of half annulus";
          parameter SI.Length t_res = 0.002 
        "Residual thickness of half annulus";
          SI.Reluctance R_mag "Magnetic reluctance";
      
          annotation (
            Coordsys(
              extent=[-100, -100; 100, 100],
              grid=[2, 2],
              component=[20, 20]),
            Window(
              x=0.16,
              y=0.15,
              width=0.6,
              height=0.6),
            Icon(
              Rectangle(extent=[-70, 30; 70, -30], style(
                  color=45,
                  fillColor=7,
                  fillPattern=1)),
              Line(points=[-90, 0; -70, 0], style(color=45)),
              Line(points=[70, 0; 90, 0], style(color=45)),
              Text(
                extent=[-100, -78; 100, -36],
                string="%name",
                style(color=45))),
            Diagram);
        equation 
          R_mag = (d/(t+t_res) + 1) / (Modelica.Constants.mue_0 * r * 4);
          R_mag*Phi = V_mag;
      
        end VariableCircularHalfAnnulus;
    
        model HollowCylinderRadialFlux 
      "Magnetic reluctance of a hollow cylinder with radial magnetic flux" 
      
          extends Interfaces.MagneticFluxTube;
      
          parameter SI.Length d_out "Outer diameter of hollow cylinder";
          parameter SI.Length d_in "Inner diameter of hollow cylinder";
          parameter SI.Length l "Length of hollow cylinder in axial direction";
          SI.Reluctance R_mag "Magnetic reluctance";
      
          annotation (
            Coordsys(
              extent=[-100, -100; 100, 100],
              grid=[2, 2],
              component=[20, 20]),
            Window(
              x=0.16,
              y=0.15,
              width=0.6,
              height=0.6),
            Icon(
              Rectangle(extent=[-70, 30; 70, -30], style(
                  color=45,
                  fillColor=7,
                  fillPattern=1)),
              Line(points=[-90, 0; -70, 0], style(color=45)),
              Line(points=[70, 0; 90, 0], style(color=45)),
              Text(
                extent=[-100, -78; 100, -36],
                string="%name",
                style(color=45))),
            Diagram);
        equation 
          R_mag = Modelica.Math.log(d_out/d_in) / (2 * Modelica.Constants.pi * Modelica.Constants.mue_0 * l);
          R_mag*Phi = V_mag;
      
        end HollowCylinderRadialFlux;
    
    end Air;
  end Reluctance;


  package Sources "Sources of a magnetic potential difference" 
    extends Modelica.Icons.Library;
    package SI = Modelica.SIunits;
  
    annotation (Coordsys(
        extent=[0, 0; 440, 456],
        grid=[2, 2],
        component=[20, 20]), Window(
        x=0.45,
        y=0.01,
        width=0.35,
        height=0.49,
        library=1,
        autolayout=1));
  
    model ConstantMagnetomotiveForce 
    "Source of constant magnetic potential difference" 
      extends Magnetic.Interfaces.MagneticFluxTube;
      parameter SI.Current theta = 1000 
      "Magnetomotive force (e.g. for a coil: current I * number of turns)";
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Icon(
          Ellipse(extent=[-40, -40; 40, 40], style(color=45)),
          Line(points=[100, 0; 40, 0], style(color=45)),
          Line(points=[-40, 0; -100, 0], style(color=45)),
          Text(
            extent=[-80, 84; 80, 52],
            string="%name",
            style(color=45)),
          Line(points=[-12, 14; 12, 14], style(color=45)),
          Line(points=[0, 14; 0, -14], style(color=45)),
          Line(points=[-12, -14; 12, -14], style(color=45))),
        Diagram(
          Line(points=[-110, 20; -85, 20], style(color=9, fillColor=9)),
          Polygon(points=[-95, 23; -85, 20; -95, 17; -95, 23], style(
              color=9,
              fillColor=9,
              fillPattern=1)),
          Line(points=[90, 20; 115, 20], style(color=9, fillColor=9)),
          Text(
            extent=[-110, 25; -90, 45],
            string="Phi",
            style(color=9)),
          Polygon(points=[105, 23; 115, 20; 105, 17; 105, 23], style(
              color=9,
              fillColor=9,
              fillPattern=1)),
          Text(
            extent=[90, 45; 110, 25],
            string="Phi",
            style(color=9)),
          Line(points=[-110, 20; -85, 20], style(color=9, fillColor=9)),
          Polygon(points=[-95, 23; -85, 20; -95, 17; -95, 23], style(
              color=9,
              fillColor=9,
              fillPattern=1)),
          Line(points=[90, 20; 115, 20], style(color=9, fillColor=9)),
          Line(points=[-125, 0; -115, 0], style(color=9)),
          Line(points=[-120, -5; -120, 5], style(color=9)),
          Text(
            extent=[-110, 25; -90, 45],
            string="Phi",
            style(color=9)),
          Polygon(points=[105, 23; 115, 20; 105, 17; 105, 23], style(
              color=9,
              fillColor=9,
              fillPattern=1)),
          Line(points=[115, 0; 125, 0], style(color=9)),
          Text(
            extent=[90, 45; 110, 25],
            string="Phi",
            style(color=9))),
        Window(
          x=0.48,
          y=0.25,
          width=0.6,
          height=0.6));
    equation 
      V_mag = theta;
    end ConstantMagnetomotiveForce;
  
    model SignalMagnetomotiveForce 
    "Signal controlled source of a magnetic potential difference" 
    
      extends Magnetic.Interfaces.MagneticFluxTube;
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Window(
          x=0.27,
          y=0.04,
          width=0.61,
          height=0.73),
        Icon(
          Ellipse(extent=[-30, 30; 30, -30], style(color=45)),
          Line(points=[-100, 0; -30, 0], style(color=45)),
          Line(points=[30, 0; 100, 0], style(color=45)),
          Line(points=[0, 10; 0, -10], style(color=45)),
          Line(points=[-10, -10; 10, -10], style(color=45)),
          Line(points=[-10, 10; 10, 10], style(color=45)),
          Text(
            extent=[-64, 78; 64, 40],
            string="%name",
            style(color=45))));
      Modelica.Blocks.Interfaces.RealInput theta 
        annotation (extent=[-10, -50; 10, -30], rotation=90);
    equation 
      V_mag =theta;
    end SignalMagnetomotiveForce;
  
    model ElectroMagneticConverter 
    
      SI.Voltage v "Voltage drop over the ideal coil";
      SI.Current i "Current through the coil";
      SI.MagneticPotentialDifference V_mag "Magnetic potential difference";
      SI.MagneticFlux Phi "Magnetic flux through the converter";
      parameter Real turns = 1000 "Number of turns";
    
      annotation (
        Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]),
        Diagram(
          Polygon(points=[-134, 63; -124, 60; -134, 57; -134, 63], style(
              color=9,
              fillColor=9,
              fillPattern=1)),
          Line(points=[-150, 60; -125, 60], style(color=9, fillColor=9)),
          Polygon(points=[141, -57; 151, -60; 141, -63; 141, -57], style(
              color=9,
              fillColor=9,
              fillPattern=1)),
          Line(points=[125, -60; 150, -60], style(color=9, fillColor=9)),
          Text(
            extent=[128, -56; 144, -41],
            string="Phi",
            style(color=9, fillColor=9)),
          Text(
            extent=[128, 64; 145, 79],
            string="Phi",
            style(
              color=9,
              gradient=2,
              fillColor=9)),
          Line(points=[-150, -59; -125, -59], style(color=9, fillColor=9)),
          Polygon(points=[-140, -56; -150, -59; -140, -62; -140, -56], style(
              color=9,
              fillColor=9,
              fillPattern=1)),
          Text(
            extent=[-141, -56; -124, -41],
            string="i",
            style(color=9, fillColor=9)),
          Text(
            extent=[-150, 63; -133, 78],
            string="i",
            style(color=9, fillColor=9)),
          Line(points=[124, 61; 149, 61], style(color=9, fillColor=9)),
          Polygon(points=[134, 64; 124, 61; 134, 58; 134, 64], style(
              color=9,
              fillColor=9,
              fillPattern=1))),
        Window(
          x=0.26,
          y=0.14,
          width=0.58,
          height=0.58),
        Icon(
          Ellipse(extent=[-58, 0; -38, 20]),
          Line(points=[-48, 60; -48, 40]),
          Ellipse(extent=[-58, 20; -38, 40]),
          Ellipse(extent=[-58, -20; -38, 0]),
          Ellipse(extent=[-58, -40; -38, -20]),
          Line(points=[-48, -40; -48, -60]),
          Rectangle(extent=[-62, 40; -48, -40], style(color=7, fillColor=7)),
          Line(points=[-48, 60; -100, 60]),
          Line(points=[-48, -60; -100, -60]),
          Line(points=[0, 100; -100, 100], style(pattern=2)),
          Line(points=[-100, 100; -100, -100], style(color=77, pattern=2)),
          Line(points=[0, -100; -100, -100], style(pattern=2)),
          Line(points=[100, 100; 0, 100], style(
              color=45,
              pattern=2,
              fillColor=45)),
          Line(points=[100, -100; 0, -100], style(color=45, pattern=2)),
          Line(points=[100, 100; 100, -100], style(
              color=45,
              pattern=2,
              fillColor=45)),
          Ellipse(extent=[12, -34; 80, 34], style(color=45)),
          Line(points=[46, -60; 46, -34], style(color=45)),
          Line(points=[34, 0; 58, 0], style(color=45)),
          Line(points=[58, 10; 58, -12], style(color=45)),
          Line(points=[46, 34; 46, 60], style(color=45)),
          Line(points=[46, 60; 100, 60], style(color=45)),
          Line(points=[46, -60; 100, -60], style(color=45)),
          Text(
            extent=[-100, 140; 100, 100],
            string="%name",
            style(color=45)),
          Line(points=[34, 10; 34, -12], style(color=45))));
      Modelica.Electrical.Analog.Interfaces.PositivePin p_el 
        annotation (extent=[-110, 50; -90, 70]);
      Modelica.Electrical.Analog.Interfaces.NegativePin n_el 
        annotation (extent=[-110, -70; -90, -50]);
      Interfaces.PositiveMagneticPort p_mag annotation (extent=[90, 50; 110, 70]);
      Interfaces.NegativeMagneticPort n_mag annotation (extent=[90, -70; 110, -50]);
    
    equation 
      v = p_el.v - n_el.v;
      i = p_el.i;
      0 = p_el.i + n_el.i;
    
      V_mag = p_mag.V_mag - n_mag.V_mag;
      Phi = p_mag.Phi;
      0 = p_mag.Phi + n_mag.Phi;
    
      V_mag = i * turns;
      turns * der(Phi)  = - v;
    
    end ElectroMagneticConverter;
  
    model PermanentMagnet 
    "Permanent Magnet with linear demagnetization curve B(H)" 
    
      annotation (Coordsys(
          extent=[-100, -100; 100, 100],
          grid=[2, 2],
          component=[20, 20]), Icon(
          Ellipse(extent=[-66, 38; 68, -46], style(color=45)),
          Text(
            extent=[-56, 24; -30, -34],
            string="N",
            style(color=45)),
          Text(
            extent=[34, 22; 60, -32],
            string="S",
            style(color=45)),
          Text(
            extent=[-78, 84; 82, 52],
            string="%name",
            style(color=45)),
          Line(points=[-66, -2; -94, -2], style(color=45)),
          Line(points=[94, -2; 68, -2], style(color=45))),
        Diagram);
      Magnetic.Reluctance.LinearReluctance R_magPM(
        l=l_PM,
        A=A_PM,
        my_r=B_r/(Modelica.Constants.mue_0*H_c)) 
        annotation (extent=[10,-20; 50,20]);
      ConstantMagnetomotiveForce theta_PM(theta=H_c*l_PM) 
        annotation (extent=[-50,-20; -10,20]);
      Interfaces.PositiveMagneticPort p annotation (extent=[-110,-10; -90,10]);
      Interfaces.NegativeMagneticPort n annotation (extent=[90,-10; 110,10]);
    
     parameter SI.Length l_PM = 0.01 "Length of permanent magnet";
      parameter SI.Area A_PM = 1e-4 "Cross sectional area of permanent magnet";
      parameter SI.MagneticFieldStrength H_c=8e+5 "Coercivity";
      parameter SI.MagneticFluxDensity B_r=1.2 "Residual Induction - Remanence";
    equation 
      connect(theta_PM.n, R_magPM.p) 
                                   annotation (points=[-10,0; 10,0], style(
            color=45, rgbcolor={255,127,0}));
    
      connect(theta_PM.p, p) annotation (points=[-50,0; -100,0], style(color=45,
            rgbcolor={255,127,0}));
      connect(R_magPM.n, n) annotation (points=[50,0; 100,0], style(color=45,
            rgbcolor={255,127,0}));
    end PermanentMagnet;
  
  end Sources;


  package Material 
  "Magnetic properties of selected soft and hard magnetic materials" 
     extends Modelica.Icons.Library;
    package Softmagnetic 
      function Steel_9SMn28K "Free cutting steel 9SMn28K" 
        extends Modelica.Icons.Function;
      
        input Real B;
        output Real my_r;
    protected 
        parameter Real myInit=6.875;
        parameter Real ca=2205;
        parameter Real cb=7.65;
        parameter Real n=7.6025;
        parameter Real BmyMax=1.051;
        Real BN;
      
        annotation (Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]), Window(
            x=0.22,
            y=0.07,
            width=0.6,
            height=0.6), 
        Documentation(info="<html>
<br>
<p>
Source of the underlying data points <i>B(H)</i>:
<dd>
<p>Measurement at Dresden University of Technology</p>
</dd>
</p>
</html>"));
      algorithm 
        BN := abs(B/BmyMax);
        my_r := (myInit + ca*BN)/(1 + cb*BN + BN^n);
      end Steel_9SMn28K;
      extends Modelica.Icons.Library;
    
    
      function VacoferS2 "Pure iron RFe20 (99.95% Fe)" 
        extends Modelica.Icons.Function;
        input Real B;
        output Real my_r;
    protected 
        Real BN;
        final parameter Real myInit=3104;
        final parameter Real ca=190300;
        final parameter Real BmyMax=1.14;
        final parameter Real cb=4;
        final parameter Real n=16.4;
      
      algorithm 
        BN := abs(B/BmyMax);
        my_r := (myInit+ca*BN)/(1+cb*BN+BN^n);
      annotation (Documentation(info="<html>
<br>
<p>
Source of the underlying data points <i>B(H)</i>:
<dd>
<p><i>Boll, R.</i>: Weichmagnetische Werkstoffe: Einf&uuml;hrung in den Magnetismus, VAC-Werkstoffe und ihre Anwendungen. 4. Aufl. Berlin, M&uuml;nchen: Siemens Aktiengesellschaft 1990, p. 287</p>
</dd>
</p>
</html>"));
      end VacoferS2;
    
    
      function MUMETALL "Nickel Iron 77% NiFe" 
        extends Modelica.Icons.Function;
        input Real B;
        output Real my_r;
    protected 
        Real BN;
        final parameter Real myInit=41939;
        final parameter Real ca=593000;
        final parameter Real BmyMax=0.48;
        final parameter Real cb=1.5;
        final parameter Real n=8.96;
      
      algorithm 
        BN := abs(B/BmyMax);
        my_r := (myInit+ca*BN)/(1+cb*BN+BN^n);
      annotation (Documentation(info="<html>
<br>
<p>
Source of the underlying data points <i>B(H)</i>:
<dd>
<p><i>Boll, R.</i>: Weichmagnetische Werkstoffe: Einf&uuml;hrung in den Magnetismus, VAC-Werkstoffe und ihre Anwendungen. 4. Aufl. Berlin, M&uuml;nchen: Siemens Aktiengesellschaft 1990, p. 287</p>
</dd>
</p>
</html>"));
      end MUMETALL;
    
    
    
      function Steel_X6Cr17 "sheet steel" 
        extends Modelica.Icons.Function;
        input Real B;
        output Real my_r;
    protected 
        Real BN;
        final parameter Real myInit=274;
        final parameter Real ca=970;
        final parameter Real BmyMax=1.1;
        final parameter Real cb=1.2;
        final parameter Real n=8.3;
      
      algorithm 
        BN := abs(B/BmyMax);
        my_r := (myInit+ca*BN)/(1+cb*BN+BN^n);
      annotation (Documentation(info="<html>
<br>
<p>
Source of the underlying data points <i>B(H)</i>:
<dd>
<p><i>Kallenbach, E.; Eick, R.; Quendt, P.; Str&ouml;hla, T.; Feindt, K.; Kallenbach, M.</i>: Elektromagnete: Grundlagen, Berechnung, Entwurf und Anwendung. 2. Aufl. Wiesbaden: B.G. Teubner 2003</p>
</dd>
</p>
</html>", revisions="<html>

</html>"));
      end Steel_X6Cr17;
    
      function M330_50A "electric plate" 
        extends Modelica.Icons.Function;
        input Real B;
        output Real my_r;
    protected 
        Real BN;
        final parameter Real myInit=514.5;
        final parameter Real ca=24250;
        final parameter Real BmyMax=0.7;
        final parameter Real cb=9.5;
        final parameter Real n=9.6;
      
      algorithm 
        BN := abs(B/BmyMax);
        my_r := (myInit+ca*BN)/(1+cb*BN+BN^n);
      annotation (Documentation(info="<html>
<br>
<p>
Source of the underlying data points <i>B(H)</i>:
<dd>
<p>Measurement at Dresden University of Technology</p>
</dd>
</p>
</html>"));
      end M330_50A;
    
    
    
    
    
    end Softmagnetic;
  annotation (Documentation(info="<html>
<br>
<p>
Currently, the relationship <i><font face=\"Symbol\">m</font><sub>r</sub> = f(B)</i> for selected ferromagnetic materials is approximated with a function. A description of that function and its coefficients can be found in:
<dd>
<p><i>Roschke, Th.</i>: Entwurf geregelter elektromagnetischer Antriebe f&uuml;r Luftsch&uuml;tze. D&uuml;sseldorf: VDI-Verlag 2000.</p>
</dd>
</p>
</html>"));
  end Material;


  package Solenoid "Reluctance type electromagnetic actuators " 
    extends Modelica.Icons.Library;
    package SI = Modelica.SIunits;
  
    model STA_195205_129 "Tubular Solenoid, Manufacturer: Saia-Burgess" 
      // Yoke Geometry
      parameter SI.Length l_yoke = 39e-3 "Total length of yoke";
      parameter SI.Length d_yoke = 19.15e-3 
      "Outer diameter of yoke resp. Solenoid";
      parameter SI.Length t_yoke = 1e-3 "Thickness of yoke";
      parameter SI.Length d_lidBore = 9.5e-3 "Diameter of lid bore";
      parameter SI.Length t_lid = 2e-3 "Thickness of yoke lids";
      parameter SI.CrossSection A_yoke = Modelica.Constants.pi * (d_yoke*d_yoke - di_yoke*di_yoke) / 4 
      "Cross sectional area of yoke";
  protected 
      parameter SI.Length di_yoke = d_yoke - 2*t_yoke "Inner yoke diameter";
    
      // Armature Geometry
  public 
      parameter SI.Length d_arm = 7.85e-3 "Outer diameter of armature";
      parameter SI.Length d_armBore = 2.39e-3 "Diameter of pushing rod bore";
      parameter SI.CrossSection A_arm = Modelica.Constants.pi * d_arm * d_arm / 4 
      "Cross sectional area of armature";
      parameter SI.Mass m_arm = 0.0133 "Armature weight";
    
      // Pole Geometry, due to radial clearance between armature pushing rod and joke slightly different from armature end face geometry
      parameter SI.Length d_pole = 8.15e-3 "Outer diameter of pole";
      parameter SI.Length d_poleBore = 3.45e-3 
      "Diameter of pole bore for armature pushing rod";
      parameter SI.CrossSection A_pole = Modelica.Constants.pi * (d_pole*d_pole - d_poleBore*d_poleBore) / 4 
      "Cross sectional area of pole";
    
      Magnetic.Reluctance.MagneticGround MagGround 
                                           annotation (extent=[-56,-40; -36,-20]);
      annotation (Diagram, Icon(
          Rectangle(extent=[-90,90; -76,-90],   style(color=45, fillColor=45)),
          Rectangle(extent=[66,92; 80,-90],   style(color=45, fillColor=45)),
          Rectangle(extent=[-90,100; 80,90],   style(color=45, fillColor=45)),
          Rectangle(extent=[-90,-90; 80,-100],   style(color=45, fillColor=45)),
          Rectangle(extent=[-30,40; 100,-40],   style(pattern=0, fillColor=45)),
          Rectangle(extent=[-88,40; -48,-40],   style(
              color=45,
              pattern=0,
              fillColor=45)),
          Rectangle(extent=[-76,90; 66,40], style(
              pattern=0,
              fillColor=7,
              rgbfillColor={255,255,255})),
          Rectangle(extent=[-76,-40; 66,-90], style(
              pattern=0,
              fillColor=7,
              rgbfillColor={255,255,255})),
          Rectangle(extent=[-48,40; -30,-40], style(
              pattern=0,
              fillColor=7,
              rgbfillColor={255,255,255})),
          Text(
            extent=[-100,140; 100,100],
            style(color=3, rgbcolor={0,0,255}),
            string="%name")),
        experiment(StopTime=100, Tolerance=1e-008),
        Documentation(info="<html>
<br>
<p>
This exemplary model describes an industrial lifting magnet: the tubular solenoid STA 195205-129 of the manufacturer <a href=\"http://www.saia-burgess.com/\">Saia-Burgess</a>. Technical data of this actuator can be found in the manufacturers internet catalogue.The graphical representation of the Modelica model is shown below:<br>
</p>
<dl>
<dd>
<p><IMG SRC=\"../Images/graphical_model_sample_solenoid.png\" ALT=\"Graphical representation of the solenoid model\"></p>
</dd>
</dl>
<br>
The magnetic network of that solenoid was derived with the assistance of magnetostatic Finite Element Analysis at different armature positions <i>x</i> for a given current <i>i</i>:<br>
<br>
<dl>
<dd>
<p><IMG SRC=\"../Images/network_over_field_plot_sample_solenoid.png\" ALT=\"Magnetic network over field plot obtained from FEA\"></p>
</dd>
</dl>
<br>
</p>
</html>"));
      Magnetic.Reluctance.NonlinearReluctance R_mFeYoke(l=5/3*l_yoke +
            d_yoke, A=A_yoke,
        redeclare function my_r_Material = 
            Magnetic.Material.Softmagnetic.Steel_9SMn28K) "Yoke reluctance" 
                                 annotation (extent=[24,10; 44,30]);
      Magnetic.Reluctance.WorkingAirGap R_mAirGap(A=A_pole) 
      "Reluctance of working air gap" 
        annotation (extent=[42,-10; 62,10],
                                         rotation=270);
    
      // Armature stroke
      parameter SI.Length x_min = 0.25e-3 
      "Minimal armature position (equals thickness of impact cushion)";
      parameter SI.Length x_max = 8e-3 "Maximum armature position";
      SI.Length x_arm( start = x_max) 
      " Armature position, identical with length of working air gap";
      Magnetic.Solenoid.Utilities.ArmatureMechanics Armature(
        limit_xMin(
        s0 =          x_min),
        limit_xMax(
        s0 =          x_max),
        ArmatureMass(s=x_arm, m=m_arm))    annotation (extent=[66,-10; 86,10]);
    
      Magnetic.Sources.ElectroMagneticConverter ElMagConverter(turns=n_turns) 
        annotation (extent=[-72,-10; -52,10]);
      Modelica.Electrical.Analog.Basic.Resistor R_coil(R=R_coil_20) 
        annotation (extent=[-90,20; -70,40], rotation=270);
      Modelica.Electrical.Analog.Interfaces.PositivePin pin_p 
        annotation (extent=[-110,50; -90,70]);
      Modelica.Electrical.Analog.Interfaces.NegativePin pin_n 
        annotation (extent=[-110,-70; -90,-50]);
    
      // Electrical Parameters
      parameter SI.Resistance R_coil_20 = 12.5 "Winding resistance at 20 C";
      parameter Real n_turns = 1146 "Number of winding turns";
    
      Magnetic.Reluctance.Air.CircularHalfAnnulus R_mStray(
        d=l_yoke/3,
        t=l_yoke/3,
        r=(d_yoke - d_arm)/2) "Coil stray reluctance" 
                 annotation (extent=[-22,-10; -2,10],
                                                    rotation=270);
      Magnetic.Reluctance.NonlinearReluctance R_mFeArm(A=A_arm, l=l_yoke
            /3,
        redeclare function my_r_Material = 
            Magnetic.Material.Softmagnetic.Steel_9SMn28K) "Armature reluctance"
                      annotation (extent=[-38,10; -18,30]);
      Magnetic.Reluctance.Air.HollowCylinderRadialFlux R_mAirParasitic(
        d_in=d_arm,
        d_out=d_lidBore,
        l=t_lid) "Parasitic airgap of armature slide guiding" 
                    annotation (extent=[-6,10; 14,30], rotation=0);
    
      // Magnetic force as alias for convenient plotting etc. (Force is calculated in working air gap R_mAirGap)
      SI.Force F_mag "Actuator force";
    
      Modelica.Mechanics.Translational.Interfaces.Flange_a ArmatureFlange 
        annotation (extent=[90,-10; 110,10]);
    equation 
      connect(MagGround.p, R_mAirGap.n)         annotation (points=[-46,-20; 52,
            -20; 52,-10],
                   style(color=45, rgbcolor={255,127,0}));
      connect(ElMagConverter.n_mag, MagGround.p)                  annotation (
          points=[-52,-6; -46,-6; -46,-20], style(color=45, rgbcolor={255,127,0}));
      connect(R_coil.p, pin_p) annotation (points=[-80,40; -80,60; -100,60],
          style(color=3, rgbcolor={0,0,255}));
      connect(R_coil.n, ElMagConverter.p_el)            annotation (points=[-80,
            20; -80,6; -72,6], style(color=3, rgbcolor={0,0,255}));
      connect(ElMagConverter.n_el, pin_n)            annotation (points=[-72,-6;
            -80,-6; -80,-60; -100,-60], style(color=3, rgbcolor={0,0,255}));
      connect(R_mStray.n, MagGround.p)              annotation (points=[-12,-10; 
          -12,-20; -46,-20],  style(color=45, rgbcolor={255,127,0}));
      connect(R_mFeArm.p, ElMagConverter.p_mag)              annotation (points=[
            -38,20; -46,20; -46,6; -52,6],
                                   style(color=45, rgbcolor={255,127,0}));
      connect(R_mAirGap.AirGapFlange, Armature.flange_a) 
        annotation (points=[55,-5.51073e-016; 64.5,-5.51073e-016; 64.5,0; 66,0],
          style(color=58, rgbcolor={0,127,0}));
      connect(R_mStray.p, R_mAirParasitic.p)        annotation (points=[-12,10; 
          -12,20; -6,20],    style(color=45, rgbcolor={255,127,0}));
      connect(R_mAirParasitic.p, R_mFeArm.n)   annotation (points=[-6,20; -18,
            20], style(color=45, rgbcolor={255,127,0}));
      connect(R_mAirParasitic.n, R_mFeYoke.p)   annotation (points=[14,20; 24,
            20], style(color=45, rgbcolor={255,127,0}));
      connect(R_mFeYoke.n, R_mAirGap.p)     annotation (points=[44,20; 52,20;
            52,10], style(color=45, rgbcolor={255,127,0}));
      F_mag = R_mAirGap.AirGapFlange.f;
    
      connect(Armature.flange_b, ArmatureFlange) 
        annotation (points=[86,0; 100,0], style(color=58, rgbcolor={0,127,0}));
    end STA_195205_129;
  
    package Utilities 
    "Model components for the mechanical subsystem of solenoid actuators" 
      class Stopper 
      "1D translational spring damper combination with gap and nonlinear force function" 
      
        extends Modelica.Mechanics.Translational.Interfaces.Compliant;
        parameter SI.Position s_rel0=0 "unstretched spring length";
        parameter Real c(
          final unit="N/m",
          final min=0) = 1e6 "spring constant";
        parameter Real d(
          final unit="N/ (m/s)",
          final min=0) = 2e2 "damping constant";
        SI.Velocity v_rel "relative velocity between flange L and R";
        Boolean Contact "false, if s_rel > 0 ";
        parameter SI.Position s_n=1e-6 "normalized entering depth";
        parameter Real n=1 "exponent for force function (n>=1)";
        SI.Velocity delta_v "equals relative velocity for movement ";
        SI.Position delta_s "equals (s_rel - s_rel0)";
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Line(points=[-100, 0; -50, 0], style(color=58)),
            Line(points=[-48, 34; -48, -46], style(color=0, thickness=4)),
            Line(points=[8, 40; 8, 2], style(color=0)),
            Line(points=[-2, 0; 38, 0; 38, 44; -2, 44], style(color=0)),
            Line(points=[38, 22; 72, 22], style(color=0)),
            Line(points=[-12, -38; -12, 20], style(color=0, thickness=4)),
            Line(points=[-12, 22; 8, 22], style(color=0)),
            Line(points=[-12, -38; -2, -38], style(color=0)),
            Line(points=[-2, -38; 10, -28; 22, -48; 38, -28; 50, -48; 64, -28; 72,
                  -40], style(color=0)),
            Rectangle(extent=[8, 44; 38, 0], style(
                color=0,
                fillColor=8,
                fillPattern=1)),
            Text(
              extent=[-28, -80; 12, -55],
              string="s_rel",
              style(color=3)),
            Line(points=[72, 0; 90, 0], style(color=58)),
            Line(points=[72, 22; 72, -42], style(color=0)),
            Line(points=[-98, -60; 98, -60], style(color=0)),
            Line(points=[-100, -29; -100, -61], style(color=0)),
            Line(points=[100, -61; 100, -28], style(color=0)),
            Polygon(points=[-101, -60; -96, -59; -96, -61; -101, -60], style(
                color=0,
                fillColor=0,
                fillPattern=1)),
            Polygon(points=[100, -60; 95, -61; 95, -59; 100, -60], style(
                color=0,
                fillColor=0,
                fillPattern=1))),
          Icon(
            Line(points=[-100, 0; -50, 0], style(color=58)),
            Line(points=[-48, 34; -48, -46], style(color=0, thickness=4)),
            Line(points=[22, 40; 22, 2], style(color=0)),
            Line(points=[12, 0; 52, 0; 52, 44; 12, 44], style(color=0)),
            Line(points=[52, 22; 72, 22], style(color=0)),
            Line(points=[-12, -38; -12, 20], style(color=0, thickness=4)),
            Line(points=[-12, 22; 22, 22], style(color=0)),
            Line(points=[-12, -38; -2, -38], style(color=0)),
            Line(points=[74, 0; 100, 0], style(color=58)),
            Line(points=[72, 22; 72, -42], style(color=0)),
            Line(points=[-2, -38; 10, -28; 22, -48; 38, -28; 50, -48; 64, -28; 72,
                  -40], style(color=0)),
            Rectangle(extent=[22, 44; 52, 0], style(
                color=0,
                fillColor=8,
                fillPattern=1)),
            Text(extent=[0, 120; 0, 60], string="%name"),
            Polygon(points=[50, -90; 20, -80; 20, -100; 50, -90], style(color=10,
                  fillColor=10)),
            Line(points=[-60, -90; 20, -90], style(color=0, fillColor=10)),
            Line(points=[12, -12; 54, 58], style(color=85)),
            Polygon(points=[46, 52; 54, 58; 52, 48; 46, 52], style(color=85,
                  fillColor=85)),
            Line(points=[20, -62; 50, -14], style(color=85, fillColor=86)),
            Polygon(points=[44, -18; 52, -12; 50, -22; 44, -18], style(color=85,
                  fillColor=85))),
          Window(
            x=0.04,
            y=0.01,
            width=0.6,
            height=0.83),
          Documentation(info="<html>
<p>
Th. Hegewald, TU Dresden 2001
</p>
<p>
Mathematical model describing the impact based on:<br>
Kamusella, A.: USAN-Handbuch. Institut fuer Feinwerktechnik:
Technische Universitaet Dresden 1999
</p>
<p>
Modelica implementation of the above model based on
Modelica.Mechanics.Translational.ElastoGap
</p>
</html>"));
      equation 
      
        v_rel = der(s_rel);
        Contact = s_rel < s_rel0;
        delta_s = abs(s_rel - s_rel0);
        delta_v = if (v_rel < 0) then v_rel else 0;
        f = if Contact then ((c*(s_rel - s_rel0) + d*delta_v)*(delta_s/s_n)^n) else 
                0;
      end Stopper;
    
      model ArmatureMechanics 
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Window(
            x=0.21,
            y=0.35,
            width=0.53,
            height=0.73),
          Diagram(
            Line(points=[-90, 84; -90, 62], style(color=9, thickness=2)),
            Line(points=[-90, 80; -50, 80], style(
                color=9,
                thickness=2,
                arrow=1)),
            Text(
              extent=[-94, 96; -86, 86],
              string="0",
              style(
                color=9,
                pattern=2,
                thickness=2)),
            Text(
              extent=[-58, 92; -50, 82],
              style(
                color=9,
                pattern=2,
                thickness=2),
              string="x")),
          Icon(
            Rectangle(extent=[-90, 10; 90, -10], style(gradient=2, fillColor=45)),
            Text(
              extent=[-100, 80; 100, 40],
              string="%name",
              style(color=45)),
            Rectangle(extent=[-70, 30; 70, -30], style(
                color=45,
                gradient=2,
                fillColor=45))),
          Documentation(info="<html>

<p>
The mechanical subsystem of a translational actuator with limited stroke is described by a sliding mass that can move between stoppers at the maximum respectively minimum position.
</p>
</html>"));
        Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b 
          annotation (extent=[90, -10; 110, 10]);
        Modelica.Mechanics.Translational.Fixed limit_xMin(s0=0) 
          annotation (extent=[-80, 20; -60, 40]);
        Modelica.Mechanics.Translational.SlidingMass ArmatureMass(m=1) 
          annotation (extent=[-10, 20; 10, 40]);
        Magnetic.Solenoid.Utilities.Stopper Stopper_xMax(
          c=1e6,
          d=2e2,
          s_n=10e-6) annotation (extent=[30, 20; 50, 40]);
        Magnetic.Solenoid.Utilities.Stopper Stopper_xMin(
          c=1e6,
          d=2e2,
          s_n=10e-6) annotation (extent=[-50, 20; -30, 40]);
        Modelica.Mechanics.Translational.Fixed limit_xMax(s0=10e-3) 
          annotation (extent=[60, 20; 80, 40]);
        Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a 
          annotation (extent=[-110,-10; -90,10]);
      equation 
        connect(ArmatureMass.flange_b, flange_b) 
          annotation (points=[10, 30; 10, 0; 100, 0]);
        connect(Stopper_xMin.flange_a, limit_xMin.flange_b) 
          annotation (points=[-50, 30; -70, 30]);
        connect(Stopper_xMin.flange_a, limit_xMin.flange_b) 
          annotation (points=[-50, 30; -70, 30]);
        connect(ArmatureMass.flange_a, Stopper_xMin.flange_b) 
          annotation (points=[-10, 30; -30, 30], style(color=58));
        connect(limit_xMax.flange_b, Stopper_xMax.flange_b) 
          annotation (points=[70, 30; 50, 30], style(color=58));
        connect(Stopper_xMax.flange_a, ArmatureMass.flange_b) 
          annotation (points=[30, 30; 10, 30], style(color=58));
        connect(ArmatureMass.flange_a, flange_a) annotation (points=[-10,30; -10,
              0; -100,0], style(color=58, rgbcolor={0,127,0}));
      end ArmatureMechanics;
    end Utilities;
    annotation (Documentation(info="<html>
<br>
<p>
Solenoid actuators are widely used in many different applications, e.g. in locking mechanisms throughout automation and automotive engineering, in fluidic valves or in relays and switchgear. A cross-sectional view on a general purpose lifting magnet is shown in the figure below.<br>
</p>
<dl>
<dd>
<p><IMG SRC=\"../Images/structure_sample_solenoid.png\" ALT=\"Structure of the sample solenoid actuator\"></p>
</dd>
</dl>
<br>
<p>
Models of solenoid actuators based on magnetic networks are very useful for both coarse design of magnetic actuators itself (e.g. main dimensions, force-stroke charactersitics, dynamic behavior) and for system simulation together with neighboring subsystems, e.g. mechanical loads or excitation and control circuits.
</p>
</html>"));
  end Solenoid;


  package Examples "Exemplary usage of solenoid models" 
    extends Modelica.Icons.Library;
    model EnforcedMovement 
    "Quasistatic calculation of force-stroke-characteristics for the sample solenoid STA 195205-129" 
      annotation (Diagram,
          Documentation(info="<html>
<br>
<p>
With this model, the force-stroke characteristic of the sample solenoid is calculated. As is typical for measurements on real actuators, the armature position is enforced while the solenoid current is kept constant. Quasi-static enforced armature movement ensures that the current is nearly constant during simulation: Dynamic effects between the electrical and the magnetic subsystem (coupled by the electro-magnetic converter of the solenoid model) can be neglected.    
</p>
<p>
<ul>
<li>Simulate with the predefined settings for 100 s,</li>
<li>Plot STA_195205_129.F_mag versus STA_195205_129.x_arm.</li>
</ul>
</p>
<p>
In the figure below, simulation results are compared with measured data and with results obtained from magneto-static Finite Element Analysis at different armature positions:
<br>
<dl>
<dd>
<p><IMG SRC=\"../Images/diagram_Fx_measurement_FEA_network.png\" ALT=\"Force-stroke charakteristic of the sample solenoid: measurement, FEA and magnetic network model\"></p>
</dd>
</dl>
<br>
</p>
</html>"));
    
      extends Modelica.Icons.Example;
    
      Modelica.Mechanics.Translational.Position EnforcedMovement 
        annotation (extent=[10,0; 30,20],rotation=180);
      Modelica.Blocks.Sources.Ramp x_set(
        duration=100,
        height=-(8e-3 - 0.25e-3),
        offset=8e-3) annotation (extent=[70,0; 50,20], rotation=0);
      Modelica.Electrical.Analog.Sources.ConstantVoltage U_supply(V=13.2) 
      "Supply voltage"   annotation (extent=[-70,0; -50,20], rotation=270);
      Modelica.Electrical.Analog.Basic.Ground Ground1 
        annotation (extent=[-70,-30; -50,-10]);
      Magnetic.Solenoid.STA_195205_129 STA_195205_129 "Sample Actuator" 
        annotation (extent=[-30,0; -10,20]);
    equation 
      connect(x_set.y, EnforcedMovement.s_ref) 
                                             annotation (points=[49,10; 40.5,10; 
          40.5,10; 32,10],   style(
          color=74,
          rgbcolor={0,0,127},
          pattern=0,
          fillColor=7,
          rgbfillColor={255,255,255},
          fillPattern=1));
      connect(Ground1.p, U_supply.n) annotation (points=[-60,-10; -60,0], style(
            color=3, rgbcolor={0,0,255}));
      connect(EnforcedMovement.flange_b, STA_195205_129.ArmatureFlange) 
        annotation (points=[10,10; 0,10; 0,10; -10,10], style(color=58,
            rgbcolor={0,127,0}));
      connect(U_supply.p, STA_195205_129.pin_p)         annotation (points=[-60,20;
            -40,20; -40,16; -30,16], style(color=3, rgbcolor={0,0,255}));
      connect(STA_195205_129.pin_n, U_supply.n)         annotation (points=[-30,4;
            -40,4; -40,0; -60,0], style(color=3, rgbcolor={0,0,255}));
    end EnforcedMovement;
  
    model PullinStroke "Pull-in stroke of the sample solenoid STA 195205-129" 
      annotation (Diagram,
          experiment(StopTime=0.05, Tolerance=1e-008),
        Documentation(info="<html>
<br>
<p>
In this model, a voltage step of 13.2VDC at time t=0 is applied to the solenoid (The step height corresponds to a duty cycle of 50% for that particular sample actuator). The armature will be pulled in until it reaches its minimum position at a stopper. 
</p>
<p>
Simulate with the predefined settings for 0,05 s:
<ul>
<li>U_step.v = 13.2 V,</li>
<li>STA_195205_129.x_arm(start = 5e-3) m.</li>
</ul>
</p>
<p>
Plot versus time:
<ul>
<li>STA_195205_129.pin_p.i,</li>
<li>STA_195205_129.x_arm,</li>
<li>STA_195205_129.F_mag.</li>
</ul>
<br>
Watch the characteristic current drop during the pull-in movement. It is due to the increase of the  inductance during the stroke (decreasing magnetic reluctance of working air gap, increasing total magnetic flux).</p>
<p>
In the figure below, simulation results obtained with the above settings are compared with a measurement:<br>
<br>
<dl>
<dd>
<p><IMG SRC=\"../Images/diagram_it_xt_measured_simulated.png\" ALT=\"Comparison of measured and simulated pull-in stroke of the sample solenoid\"></p>
</dd>
</dl>
<br>
</p>
</html>"));
      extends Modelica.Icons.Example;
      Modelica.Electrical.Analog.Basic.Ground Ground 
        annotation (extent=[-40,-30; -20,-10]);
      Modelica.Electrical.Analog.Sources.StepVoltage U_step(V=13.2) 
        annotation (extent=[-40,0; -20,20], rotation=270);
      Solenoid.STA_195205_129 STA_195205_129(x_arm(start=5e-3)) 
        annotation (extent=[0,0; 20,20]);
    equation 
      connect(Ground.p, U_step.n)  annotation (points=[-30,-10; -30,-5; -30,0; 
          -30,0],                                                       style(
            color=3, rgbcolor={0,0,255}));
      connect(U_step.p, STA_195205_129.pin_p)   annotation (points=[-30,20; -12,
          20; -12,16; 0,16], style(color=3, rgbcolor={0,0,255}));
      connect(U_step.n, STA_195205_129.pin_n)   annotation (points=[-30,0; -12,
          0; -12,4; 0,4],           style(color=3, rgbcolor={0,0,255}));
    end PullinStroke;
  
    model MotionCycleWithLoad 
    "Pull-in and push-out stroke of a voltage-fed solenoid with mechanical load" 
      annotation (Diagram,
          experiment(StopTime=0.1, Tolerance=1e-008),
        Documentation(info="<html>
<br>
<p>
With this model, one motion cycle (pull-in stroke and pull out stroke) of a voltage-fed solenoid actuator with a mechanical load is simulated. Reluctance forces always act towards a shortening of the working air gap, since the direction of the generated magnetic force is independent on the direction of the current in the coil. In most applications of solenoid actuators, a return spring is used to deliver the energy for the return stroke.
</p>
<p>
Simulate with the predefined settings for 0.1 s:
<ul>
<li>LoadMass.s(start = 3e-3) m,</li>
<li>LoadMass.m = 0.01 kg,</li>
<li>ReturnSpring.c = 1000 N/m.</li>
</ul>
</p>
<p>
Plot versus time:
<ul>
<li>U_pulse.v,</li>
<li>STA_195205_129.pin_p.i,</li>
<li>LoadMass.s</li>
</ul>
</p>
</html>"));
      extends Modelica.Icons.Example;
      Modelica.Electrical.Analog.Basic.Ground Ground 
        annotation (extent=[-50,-30; -30,-10]);
      Solenoid.STA_195205_129 STA_195205_129(x_arm(start=3e-3), x_max=3e-3) 
        annotation (extent=[-10,0; 10,20]);
      Modelica.Electrical.Analog.Sources.PulseVoltage U_pulse(V=13.2, width=5) 
        annotation (extent=[-50,0; -30,20], rotation=270);
      Modelica.Mechanics.Translational.Spring ReturnSpring(c=1000, s_rel0=0) 
        annotation (extent=[30,0; 50,20]);
      Modelica.Mechanics.Translational.Fixed FrameWithSpringPreload(s0=4e-3) 
        annotation (extent=[40,-20; 60,0]);
      Modelica.Mechanics.Translational.SlidingMass LoadMass(m=0.01, s(start=
              3e-3)) "Mechanical Load to be moved" 
        annotation (extent=[30,30; 50,50]);
    equation 
      connect(U_pulse.p, STA_195205_129.pin_p) annotation (points=[-40,20; -20,
            20; -20,16; -10,16], style(color=3, rgbcolor={0,0,255}));
      connect(STA_195205_129.pin_n, U_pulse.n) annotation (points=[-10,4; -20,4;
            -20,0; -40,0], style(color=3, rgbcolor={0,0,255}));
      connect(Ground.p, U_pulse.n) annotation (points=[-40,-10; -40,0], style(
            color=3, rgbcolor={0,0,255}));
      connect(STA_195205_129.ArmatureFlange, ReturnSpring.flange_a) annotation (
         points=[10,10; 30,10], style(color=58, rgbcolor={0,127,0}));
      connect(ReturnSpring.flange_b, FrameWithSpringPreload.flange_b) 
        annotation (points=[50,10; 50,-10], style(color=58, rgbcolor={0,127,0}));
      connect(LoadMass.flange_a, ReturnSpring.flange_a) annotation (points=[30,
            40; 30,10], style(color=58, rgbcolor={0,127,0}));
    end MotionCycleWithLoad;
  end Examples;
end Magnetic;
